# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'design.ui'
#
# Created by: PyQt4 UI code generator 4.11.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(1181, 853)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(QtGui.QLayout.SetMaximumSize)
        self.verticalLayout.setContentsMargins(-1, -1, -1, 0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.groupBox_2 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.verticalLayout_7 = QtGui.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        self.gridLayout_2 = QtGui.QGridLayout()
        self.gridLayout_2.setObjectName(_fromUtf8("gridLayout_2"))
        self.btnBrowse = QtGui.QPushButton(self.groupBox_2)
        self.btnBrowse.setMinimumSize(QtCore.QSize(100, 0))
        self.btnBrowse.setMaximumSize(QtCore.QSize(100, 50))
        self.btnBrowse.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.btnBrowse.setObjectName(_fromUtf8("btnBrowse"))
        self.gridLayout_2.addWidget(self.btnBrowse, 1, 0, 1, 1, QtCore.Qt.AlignHCenter)
        self.horizontalLayout_8 = QtGui.QHBoxLayout()
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        self.lblInputFilename = QtGui.QLabel(self.groupBox_2)
        self.lblInputFilename.setMaximumSize(QtCore.QSize(200, 16777215))
        self.lblInputFilename.setObjectName(_fromUtf8("lblInputFilename"))
        self.horizontalLayout_8.addWidget(self.lblInputFilename, QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter)
        self.gridLayout_2.addLayout(self.horizontalLayout_8, 0, 0, 1, 1)
        self.gridLayout_2.setRowStretch(0, 10)
        self.gridLayout_2.setRowStretch(1, 7)
        self.verticalLayout_7.addLayout(self.gridLayout_2)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout_6 = QtGui.QVBoxLayout(self.groupBox)
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        self.scrollArea = QtGui.QScrollArea(self.groupBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName(_fromUtf8("scrollArea"))
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 224, 252))
        self.scrollAreaWidgetContents.setObjectName(_fromUtf8("scrollAreaWidgetContents"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_4.setContentsMargins(-1, 0, -1, 0)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.verticalLayout_3 = QtGui.QVBoxLayout()
        self.verticalLayout_3.setContentsMargins(-1, -1, -1, 0)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.checkBoxZeroZTrained = QtGui.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxZeroZTrained.setChecked(False)
        self.checkBoxZeroZTrained.setObjectName(_fromUtf8("checkBoxZeroZTrained"))
        self.verticalLayout_3.addWidget(self.checkBoxZeroZTrained)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        spacerItem = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.checkBoxKnownZ = QtGui.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxKnownZ.setObjectName(_fromUtf8("checkBoxKnownZ"))
        self.horizontalLayout.addWidget(self.checkBoxKnownZ)
        self.lineEditKnownZ = QtGui.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditKnownZ.setObjectName(_fromUtf8("lineEditKnownZ"))
        self.horizontalLayout.addWidget(self.lineEditKnownZ)
        self.horizontalLayout.setStretch(2, 1)
        self.horizontalLayout.setStretch(3, 1)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.checkBoxAgnosticZTrained = QtGui.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxAgnosticZTrained.setCheckable(True)
        self.checkBoxAgnosticZTrained.setTristate(False)
        self.checkBoxAgnosticZTrained.setObjectName(_fromUtf8("checkBoxAgnosticZTrained"))
        self.verticalLayout_3.addWidget(self.checkBoxAgnosticZTrained)
        self.checkBoxGalTrained = QtGui.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxGalTrained.setObjectName(_fromUtf8("checkBoxGalTrained"))
        self.verticalLayout_3.addWidget(self.checkBoxGalTrained)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.verticalLayout_5 = QtGui.QVBoxLayout()
        self.verticalLayout_5.setContentsMargins(-1, -1, -1, 0)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setContentsMargins(-1, 0, -1, -1)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label = QtGui.QLabel(self.scrollAreaWidgetContents)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.lineEditMinZ = QtGui.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditMinZ.setObjectName(_fromUtf8("lineEditMinZ"))
        self.gridLayout.addWidget(self.lineEditMinZ, 0, 1, 1, 1)
        self.label_2 = QtGui.QLabel(self.scrollAreaWidgetContents)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.lineEditMaxZ = QtGui.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditMaxZ.setObjectName(_fromUtf8("lineEditMaxZ"))
        self.gridLayout.addWidget(self.lineEditMaxZ, 1, 1, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout)
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.label_3 = QtGui.QLabel(self.scrollAreaWidgetContents)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.horizontalLayout_4.addWidget(self.label_3)
        self.lineEditSmooth = QtGui.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditSmooth.setMaximumSize(QtCore.QSize(40, 16777215))
        self.lineEditSmooth.setObjectName(_fromUtf8("lineEditSmooth"))
        self.horizontalLayout_4.addWidget(self.lineEditSmooth)
        self.horizontalSliderSmooth = QtGui.QSlider(self.scrollAreaWidgetContents)
        self.horizontalSliderSmooth.setMaximum(20)
        self.horizontalSliderSmooth.setProperty("value", 7)
        self.horizontalSliderSmooth.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderSmooth.setObjectName(_fromUtf8("horizontalSliderSmooth"))
        self.horizontalLayout_4.addWidget(self.horizontalSliderSmooth)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.verticalLayout_4.addLayout(self.verticalLayout_5)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_6.addWidget(self.scrollArea)
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.btnRefit = QtGui.QPushButton(self.groupBox)
        self.btnRefit.setMaximumSize(QtCore.QSize(150, 16777215))
        self.btnRefit.setObjectName(_fromUtf8("btnRefit"))
        self.horizontalLayout_6.addWidget(self.btnRefit)
        self.verticalLayout_6.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtGui.QHBoxLayout()
        self.horizontalLayout_7.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.btnCancel = QtGui.QPushButton(self.groupBox)
        self.btnCancel.setMinimumSize(QtCore.QSize(0, 0))
        self.btnCancel.setMaximumSize(QtCore.QSize(100, 16777215))
        self.btnCancel.setObjectName(_fromUtf8("btnCancel"))
        self.horizontalLayout_7.addWidget(self.btnCancel)
        self.verticalLayout_6.addLayout(self.horizontalLayout_7)
        self.verticalLayout.addWidget(self.groupBox)
        self.progressBar = QtGui.QProgressBar(self.centralwidget)
        self.progressBar.setProperty("value", 24)
        self.progressBar.setObjectName(_fromUtf8("progressBar"))
        self.verticalLayout.addWidget(self.progressBar)
        spacerItem2 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.verticalLayout.setStretch(0, 5)
        self.verticalLayout.setStretch(1, 16)
        self.verticalLayout.setStretch(3, 12)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.verticalLayout_2.setContentsMargins(-1, -1, 0, -1)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.groupBox_3 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName(_fromUtf8("groupBox_3"))
        self.verticalLayout_8 = QtGui.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        self.listWidget = QtGui.QListWidget(self.groupBox_3)
        self.listWidget.setObjectName(_fromUtf8("listWidget"))
        item = QtGui.QListWidgetItem()
        self.listWidget.addItem(item)
        self.verticalLayout_8.addWidget(self.listWidget)
        self.verticalLayout_2.addWidget(self.groupBox_3)
        self.groupBox_4 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_4.setObjectName(_fromUtf8("groupBox_4"))
        self.verticalLayout_9 = QtGui.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_9.setObjectName(_fromUtf8("verticalLayout_9"))
        self.horizontalLayout_9 = QtGui.QHBoxLayout()
        self.horizontalLayout_9.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.horizontalLayout_9.setContentsMargins(-1, -1, -1, 5)
        self.horizontalLayout_9.setObjectName(_fromUtf8("horizontalLayout_9"))
        self.label_9 = QtGui.QLabel(self.groupBox_4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.horizontalLayout_9.addWidget(self.label_9)
        self.comboBoxSNType = QtGui.QComboBox(self.groupBox_4)
        self.comboBoxSNType.setEditable(False)
        self.comboBoxSNType.setObjectName(_fromUtf8("comboBoxSNType"))
        self.horizontalLayout_9.addWidget(self.comboBoxSNType)
        self.comboBoxAge = QtGui.QComboBox(self.groupBox_4)
        self.comboBoxAge.setObjectName(_fromUtf8("comboBoxAge"))
        self.horizontalLayout_9.addWidget(self.comboBoxAge)
        self.comboBoxHost = QtGui.QComboBox(self.groupBox_4)
        self.comboBoxHost.setObjectName(_fromUtf8("comboBoxHost"))
        self.comboBoxHost.addItem(_fromUtf8(""))
        self.horizontalLayout_9.addWidget(self.comboBoxHost)
        self.horizontalLayout_9.setStretch(0, 3)
        self.horizontalLayout_9.setStretch(1, 100)
        self.horizontalLayout_9.setStretch(2, 100)
        self.horizontalLayout_9.setStretch(3, 100)
        self.verticalLayout_9.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(-1, 3, -1, -1)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.labelTemplateName = QtGui.QLabel(self.groupBox_4)
        self.labelTemplateName.setMinimumSize(QtCore.QSize(270, 0))
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setItalic(True)
        self.labelTemplateName.setFont(font)
        self.labelTemplateName.setObjectName(_fromUtf8("labelTemplateName"))
        self.horizontalLayout_2.addWidget(self.labelTemplateName)
        self.pushButtonLeftTemplate = QtGui.QPushButton(self.groupBox_4)
        self.pushButtonLeftTemplate.setObjectName(_fromUtf8("pushButtonLeftTemplate"))
        self.horizontalLayout_2.addWidget(self.pushButtonLeftTemplate)
        self.pushButtonRightTemplate = QtGui.QPushButton(self.groupBox_4)
        self.pushButtonRightTemplate.setObjectName(_fromUtf8("pushButtonRightTemplate"))
        self.horizontalLayout_2.addWidget(self.pushButtonRightTemplate)
        spacerItem3 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.label_4 = QtGui.QLabel(self.groupBox_4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.horizontalLayout_2.addWidget(self.label_4)
        self.lineEditRedshift = QtGui.QLineEdit(self.groupBox_4)
        self.lineEditRedshift.setObjectName(_fromUtf8("lineEditRedshift"))
        self.horizontalLayout_2.addWidget(self.lineEditRedshift)
        self.horizontalSliderRedshift = QtGui.QSlider(self.groupBox_4)
        self.horizontalSliderRedshift.setMaximum(10000)
        self.horizontalSliderRedshift.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderRedshift.setObjectName(_fromUtf8("horizontalSliderRedshift"))
        self.horizontalLayout_2.addWidget(self.horizontalSliderRedshift)
        self.horizontalLayout_2.setStretch(5, 1)
        self.horizontalLayout_2.setStretch(6, 6)
        self.verticalLayout_9.addLayout(self.horizontalLayout_2)
        self.graphicsView_2 = PlotWidget(self.groupBox_4)
        self.graphicsView_2.setObjectName(_fromUtf8("graphicsView_2"))
        self.verticalLayout_9.addWidget(self.graphicsView_2)
        self.graphicsView = PlotWidget(self.groupBox_4)
        self.graphicsView.setObjectName(_fromUtf8("graphicsView"))
        self.verticalLayout_9.addWidget(self.graphicsView)
        self.verticalLayout_9.setStretch(2, 1)
        self.verticalLayout_9.setStretch(3, 3)
        self.verticalLayout_2.addWidget(self.groupBox_4)
        self.verticalLayout_2.setStretch(0, 3)
        self.verticalLayout_2.setStretch(1, 10)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.horizontalLayout_3.setStretch(0, 9)
        self.horizontalLayout_3.setStretch(1, 32)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1181, 25))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        MainWindow.setMenuBar(self.menubar)
        self.statusBar = QtGui.QStatusBar(MainWindow)
        self.statusBar.setObjectName(_fromUtf8("statusBar"))
        MainWindow.setStatusBar(self.statusBar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow", None))
        self.groupBox_2.setTitle(_translate("MainWindow", "Select Spectra", None))
        self.btnBrowse.setText(_translate("MainWindow", "Browse", None))
        self.lblInputFilename.setText(_translate("MainWindow", "Select SN File...", None))
        self.groupBox.setTitle(_translate("MainWindow", "Priors", None))
        self.checkBoxZeroZTrained.setText(_translate("MainWindow", "Trained at z = 0", None))
        self.checkBoxKnownZ.setText(_translate("MainWindow", "Known z", None))
        self.checkBoxAgnosticZTrained.setText(_translate("MainWindow", "Agnostic Redshift Model", None))
        self.checkBoxGalTrained.setText(_translate("MainWindow", "SN + Gal Trained Model", None))
        self.label.setText(_translate("MainWindow", "Min Redshift", None))
        self.lineEditMinZ.setText(_translate("MainWindow", "0", None))
        self.label_2.setText(_translate("MainWindow", "Max Redshift", None))
        self.lineEditMaxZ.setText(_translate("MainWindow", "0.5", None))
        self.label_3.setText(_translate("MainWindow", "Smooth", None))
        self.lineEditSmooth.setText(_translate("MainWindow", "7", None))
        self.btnRefit.setText(_translate("MainWindow", "Re-fit with priors", None))
        self.btnCancel.setText(_translate("MainWindow", "Cancel", None))
        self.groupBox_3.setTitle(_translate("MainWindow", "Best Matches", None))
        __sortingEnabled = self.listWidget.isSortingEnabled()
        self.listWidget.setSortingEnabled(False)
        item = self.listWidget.item(0)
        item.setText(_translate("MainWindow", "Best Matches", None))
        self.listWidget.setSortingEnabled(__sortingEnabled)
        self.groupBox_4.setTitle(_translate("MainWindow", "Analyse selection", None))
        self.label_9.setText(_translate("MainWindow", "Plot Template", None))
        self.comboBoxHost.setItemText(0, _translate("MainWindow", "Host Galaxy", None))
        self.labelTemplateName.setText(_translate("MainWindow", "Template Name", None))
        self.pushButtonLeftTemplate.setText(_translate("MainWindow", "<", None))
        self.pushButtonRightTemplate.setText(_translate("MainWindow", ">", None))
        self.label_4.setText(_translate("MainWindow", "Redshift", None))
        self.lineEditRedshift.setText(_translate("MainWindow", "0", None))

from pyqtgraph import PlotWidget
